@@ -62,7 +62,6 @@
 #include "startup_notification.h"
 #include "compositor.h"
 #include "spinning_cursor.h"
-#include "stock_icons.h"
 
 #define BASE_EVENT_MASK \
     SubstructureNotifyMask|\
@@ -410,7 +409,6 @@ compositor_callback (const gchar  *name,
 static int
 initialize (gint compositor_mode, gboolean replace_wm)
 {
-    long ws;
     gint i, nscreens;
 
     TRACE ("entering initialize");
@@ -439,7 +437,6 @@ initialize (gint compositor_mode, gboolean replace_wm)
 #else /* HAVE_COMPOSITOR */
     main_display_info->enable_compositor = FALSE;
 #endif /* HAVE_COMPOSITOR */
-    initWMStockIcons ();
 
     initModifiers (main_display_info->dpy);
 
@@ -497,13 +494,7 @@ initialize (gint compositor_mode, gboolean replace_wm)
 
         sn_init_display (screen_info);
         myDisplayAddScreen (main_display_info, screen_info);
-        setGnomeProtocols (main_display_info, screen_info->xroot, screen_info->xfwm4_win);
-        setHint (main_display_info, screen_info->xroot, WIN_SUPPORTING_WM_CHECK, screen_info->xfwm4_win);
-        setHint (main_display_info, screen_info->xroot, WIN_DESKTOP_BUTTON_PROXY, screen_info->xfwm4_win);
-        setHint (main_display_info, screen_info->xfwm4_win, WIN_DESKTOP_BUTTON_PROXY, screen_info->xfwm4_win);
-        getHint (main_display_info, screen_info->xroot, WIN_WORKSPACE, &ws);
-        screen_info->current_ws = (int) ws;
-        getGnomeDesktopMargins (main_display_info, screen_info->xroot, screen_info->gnome_margins);
+        screen_info->current_ws = getNetCurrentDesktop (main_display_info, screen_info->xroot);
         setUTF8StringHint (main_display_info, screen_info->xfwm4_win, NET_WM_NAME, "Xfwm4");
         setNetSupportedHint (main_display_info, screen_info->xroot, screen_info->xfwm4_win);
         initNetDesktopInfo (main_display_info, screen_info->xroot, screen_info->current_ws,
@@ -514,7 +505,7 @@ initialize (gint compositor_mode, gboolean replace_wm)
 
         clientFrameAll (screen_info);
 
-        initGtkCallbacks (screen_info);
+        initPerScreenCallbacks (screen_info);
 
         XDefineCursor (main_display_info->dpy, screen_info->xroot, myDisplayGetCursorRoot(main_display_info));
     }
@@ -526,6 +517,7 @@ initialize (gint compositor_mode, gboolean replace_wm)
     }
     main_display_info->xfilter = eventFilterInit ((gpointer) main_display_info);
     eventFilterPush (main_display_info->xfilter, xfwm4_event_filter, (gpointer) main_display_info);
+    initPerDisplayCallbacks (main_display_info);
 
     return sessionStart (main_display_info);
 }
@@ -595,7 +587,7 @@ main (int argc, char **argv)
     switch (status)
     {
         case -1:
-            g_warning ("Another Window Manager is already running");
+            g_warning ("Could not find a screen to manage, exiting");
             exit (1);
             break;
         case -2:
