@@ -393,6 +393,7 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
 
     gchar imagename[30];
     GValue tmp_val = { 0, };
+    GValue tmp_val2 = { 0, };
     DisplayInfo *display_info;
     xfwmColorSymbol colsym[ XPM_COLOR_SYMBOL_SIZE + 1 ];
     GtkWidget *widget;
@@ -448,11 +449,11 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
 
     /* ... and distance */
     display_info->double_click_distance = abs (getIntValue ("double_click_distance", rc));
-    g_value_init (&tmp_val, G_TYPE_INT);
-    if (gdk_setting_get ("gtk-double-click-distance", &tmp_val))
+    g_value_init (&tmp_val2, G_TYPE_INT);
+    if (gdk_setting_get ("gtk-double-click-distance", &tmp_val2))
     {
-        display_info->double_click_distance = abs (g_value_get_int (&tmp_val));
-        g_value_unset (&tmp_val);
+        display_info->double_click_distance = abs (g_value_get_int (&tmp_val2));
+        g_value_unset (&tmp_val2);
     }
 
     screen_info->font_height = 0;
@@ -602,6 +603,7 @@ loadKeyBindings (ScreenInfo *screen_info)
     parseShortcut (screen_info, KEY_POPUP_MENU, "popup_menu_key", shortcuts);
     parseShortcut (screen_info, KEY_PREV_WORKSPACE, "prev_workspace_key", shortcuts);
     parseShortcut (screen_info, KEY_RAISE_WINDOW, "raise_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_RAISELOWER_WINDOW, "raiselower_window_key", shortcuts);
     parseShortcut (screen_info, KEY_RESIZE, "resize_window_key", shortcuts);
     parseShortcut (screen_info, KEY_RIGHT_WORKSPACE, "right_workspace_key", shortcuts);
     parseShortcut (screen_info, KEY_SHADE_WINDOW, "shade_window_key", shortcuts);
@@ -609,6 +611,10 @@ loadKeyBindings (ScreenInfo *screen_info)
     parseShortcut (screen_info, KEY_STICK_WINDOW, "stick_window_key", shortcuts);
     parseShortcut (screen_info, KEY_SWITCH_APPLICATION, "switch_application_key", shortcuts);
     parseShortcut (screen_info, KEY_SWITCH_WINDOW, "switch_window_key", shortcuts);
+    parseShortcut (screen_info, KEY_TILE_DOWN, "tile_down_key", shortcuts);
+    parseShortcut (screen_info, KEY_TILE_LEFT, "tile_left_key", shortcuts);
+    parseShortcut (screen_info, KEY_TILE_RIGHT, "tile_right_key", shortcuts);
+    parseShortcut (screen_info, KEY_TILE_UP, "tile_up_key", shortcuts);
     parseShortcut (screen_info, KEY_TOGGLE_ABOVE, "above_key", shortcuts);
     parseShortcut (screen_info, KEY_TOGGLE_FULLSCREEN, "fullscreen_key", shortcuts);
     parseShortcut (screen_info, KEY_UP_WORKSPACE, "up_workspace_key", shortcuts);
@@ -1033,10 +1039,6 @@ initSettings (ScreenInfo *screen_info)
     {
         workspaceSetCount (screen_info, (guint) MAX (val, 1));
     }
-    else if (getHint (display_info, screen_info->xroot, WIN_WORKSPACE_COUNT, &val))
-    {
-        workspaceSetCount (screen_info, (guint) MAX (val, 1));
-    }
 
     if (getUTF8StringList (display_info, screen_info->xroot, NET_DESKTOP_NAMES, &names, &i))
     {
