@@ -92,6 +92,7 @@ enum
     KEY_POPUP_MENU,
     KEY_PREV_WORKSPACE,
     KEY_RAISE_WINDOW,
+    KEY_RAISELOWER_WINDOW,
     KEY_RESIZE,
     KEY_RIGHT_WORKSPACE,
     KEY_SHADE_WINDOW,
@@ -99,6 +100,10 @@ enum
     KEY_STICK_WINDOW,
     KEY_SWITCH_APPLICATION,
     KEY_SWITCH_WINDOW,
+    KEY_TILE_DOWN,
+    KEY_TILE_LEFT,
+    KEY_TILE_RIGHT,
+    KEY_TILE_UP,
     KEY_TOGGLE_ABOVE,
     KEY_TOGGLE_FULLSCREEN,
     KEY_UP_WORKSPACE,
